@@ -3,8 +3,8 @@ use std::cmp::Ordering;
 use std::hash::{Hash, Hasher};
 use std::mem::{ManuallyDrop, MaybeUninit};
 use std::ops::{
-    Deref, DerefMut, Index, IndexMut, Range, RangeFrom, RangeFull, RangeInclusive, RangeTo,
-    RangeToInclusive,
+    Bound, Deref, DerefMut, Index, IndexMut, Range, RangeBounds, RangeFrom, RangeFull,
+    RangeInclusive, RangeTo, RangeToInclusive,
 };
 use std::{fmt, ptr, slice};
 
@@ -51,6 +51,71 @@ impl<'a> Drop for SetLenOnDrop<'a> {
     }
 }
 
+/// A draining iterator for `StackVec<T, N>`.
+///
+/// This `struct` is created by [`StackVec::drain()`].
+pub struct Drain<'a, T: 'a, const N: usize> {
+    /// Index of tail to preserve
+    tail_start: usize,
+    /// Length of tail
+    tail_len: usize,
+    /// Current remaining range to remove
+    iter: slice::Iter<'a, T>,
+    vec: NonNull<StackVec<T, N>>,
+}
+
+unsafe impl<'a, T: Sync, const N: usize> Sync for Drain<'a, T, N> {}
+unsafe impl<'a, T: Send, const N: usize> Send for Drain<'a, T, N> {}
+
+impl<'a, T: 'a, const N: usize> Iterator for Drain<'a, T, N> {
+    type Item = T;
+
+    #[inline]
+    fn next(&mut self) -> Option<Self::Item> {
+        self.iter
+            .next()
+            .map(|elt| unsafe { ptr::read(elt as *const _) })
+    }
+
+    #[inline]
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        self.iter.size_hint()
+    }
+}
+
+impl<'a, T: 'a, const N: usize> DoubleEndedIterator for Drain<'a, T, N> {
+    #[inline]
+    fn next_back(&mut self) -> Option<Self::Item> {
+        self.iter
+            .next_back()
+            .map(|elt| unsafe { ptr::read(elt as *const _) })
+    }
+}
+
+impl<'a, T: 'a, const N: usize> ExactSizeIterator for Drain<'a, T, N> {}
+
+impl<'a, T: 'a, const N: usize> Drop for Drain<'a, T, N> {
+    fn drop(&mut self) {
+        // len is currently 0 so panicking while dropping will not cause a double drop.
+
+        // exhaust self first
+        while let Some(_) = self.next() {}
+
+        if self.tail_len > 0 {
+            unsafe {
+                let source_vec = self.vec.as_mut();
+                // memmove back untouched tail, update to new length
+                let start = source_vec.len();
+                let tail = self.tail_start;
+                let src = source_vec.as_ptr().add(tail);
+                let dst = source_vec.as_mut_ptr().add(start);
+                ptr::copy(src, dst, self.tail_len);
+                source_vec.set_len(start + self.tail_len);
+            }
+        }
+    }
+}
+
 /// A `Vec`-like container that stores elements on the stack.
 ///
 /// The `StackVec` is a vector backed by a fixed size array. It keeps track of
@@ -447,6 +512,179 @@ impl<T, const N: usize> StackVec<T, N> {
             ptr::replace(hole, last)
         }
     }
+
+    /// Create a draining iterator that removes the specified range in the vector
+    /// and yields the removed items from start to end. The element range is
+    /// removed even if the iterator is not consumed until the end.
+    ///
+    /// Note: It is unspecified how many elements are removed from the vector,
+    /// if the `Drain` value is leaked.
+    ///
+    /// # Panics
+    /// If the starting point is greater than the end point or if
+    /// the end point is greater than the length of the vector.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use stack_buf::StackVec;
+    ///
+    /// let mut v1 = StackVec::from([1, 2, 3]);
+    /// let v2: StackVec<_, 3> = v1.drain(0..2).collect();
+    /// assert_eq!(&v1[..], &[3]);
+    /// assert_eq!(&v2[..], &[1, 2]);
+    /// ```
+    #[inline]
+    pub fn drain<R>(&mut self, range: R) -> Drain<T, N>
+    where
+        R: RangeBounds<usize>,
+    {
+        // Memory safety
+        //
+        // When the Drain is first created, it shortens the length of
+        // the source vector to make sure no uninitialized or moved-from elements
+        // are accessible at all if the Drain's destructor never gets to run.
+        //
+        // Drain will ptr::read out the values to remove.
+        // When finished, remaining tail of the vec is copied back to cover
+        // the hole, and the vector length is restored to the new length.
+        //
+        let len = self.len();
+        let start = match range.start_bound() {
+            Bound::Unbounded => 0,
+            Bound::Included(&i) => i,
+            Bound::Excluded(&i) => i.saturating_add(1),
+        };
+        let end = match range.end_bound() {
+            Bound::Excluded(&j) => j,
+            Bound::Included(&j) => j.saturating_add(1),
+            Bound::Unbounded => len,
+        };
+
+        // bounds check happens here (before length is changed!)
+        let range_slice: *const _ = &self[start..end];
+
+        // Calling `set_len` creates a fresh and thus unique mutable references, making all
+        // older aliases we created invalid. So we cannot call that function.
+        self.len = start as Size;
+
+        unsafe {
+            Drain {
+                tail_start: end,
+                tail_len: len - end,
+                iter: (*range_slice).iter(),
+                vec: NonNull::new_unchecked(self as *mut _),
+            }
+        }
+    }
+
+    /// Retains only the elements specified by the predicate.
+    ///
+    /// In other words, remove all elements `e` such that `f(&e)` returns `false`.
+    /// This method operates in place and preserves the order of the retained
+    /// elements.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use stack_buf::StackVec;
+    ///
+    /// let mut vec = StackVec::from([1, 2, 3, 4]);
+    /// vec.retain(|x| *x & 1 != 0 );
+    /// assert_eq!(&vec[..], &[1, 3]);
+    /// ```
+    #[inline]
+    pub fn retain<F>(&mut self, mut f: F)
+    where
+        F: FnMut(&mut T) -> bool,
+    {
+        let mut del = 0;
+        let len = self.len();
+        for i in 0..len {
+            if !f(&mut self[i]) {
+                del += 1;
+            } else if del > 0 {
+                self.swap(i - del, i);
+            }
+        }
+        self.truncate(len - del);
+    }
+
+    /// Removes all but the first of consecutive elements in the vector satisfying a given equality
+    /// relation.
+    ///
+    /// The `same_bucket` function is passed references to two elements from the vector and
+    /// must determine if the elements compare equal. The elements are passed in opposite order
+    /// from their order in the slice, so if `same_bucket(a, b)` returns `true`, `a` is removed.
+    ///
+    /// If the vector is sorted, this removes all duplicates.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use stack_buf::stack_vec;
+    ///
+    /// let mut vec = stack_vec!["foo", "bar", "Bar", "baz", "bar"];
+    ///
+    /// vec.dedup_by(|a, b| a.eq_ignore_ascii_case(b));
+    ///
+    /// assert_eq!(&vec[..], ["foo", "bar", "baz", "bar"]);
+    /// ```
+    pub fn dedup_by<F>(&mut self, mut same_bucket: F)
+    where
+        F: FnMut(&mut T, &mut T) -> bool,
+    {
+        // See the implementation of Vec::dedup_by in the
+        // standard library for an explanation of this algorithm.
+        let len = self.len();
+        if len <= 1 {
+            return;
+        }
+
+        let ptr = self.as_mut_ptr();
+        let mut w: usize = 1;
+
+        unsafe {
+            for r in 1..len {
+                let p_r = ptr.add(r);
+                let p_wm1 = ptr.add(w - 1);
+                if !same_bucket(&mut *p_r, &mut *p_wm1) {
+                    if r != w {
+                        let p_w = p_wm1.add(1);
+                        std::mem::swap(&mut *p_r, &mut *p_w);
+                    }
+                    w += 1;
+                }
+            }
+        }
+
+        self.truncate(w);
+    }
+
+    /// Removes all but the first of consecutive elements in the vector that resolve to the same
+    /// key.
+    ///
+    /// If the vector is sorted, this removes all duplicates.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use stack_buf::stack_vec;
+    ///
+    /// let mut vec = stack_vec![10, 20, 21, 30, 20];
+    ///
+    /// vec.dedup_by_key(|i| *i / 10);
+    ///
+    /// assert_eq!(&vec[..], [10, 20, 30, 20]);
+    /// ```
+    #[inline]
+    pub fn dedup_by_key<F, K>(&mut self, mut key: F)
+    where
+        F: FnMut(&mut T) -> K,
+        K: PartialEq,
+    {
+        self.dedup_by(|a, b| key(a) == key(b))
+    }
 }
 
 impl<T: Clone, const N: usize> StackVec<T, N> {
@@ -462,8 +700,9 @@ impl<T: Clone, const N: usize> StackVec<T, N> {
     /// use stack_buf::StackVec;
     ///
     /// let vec = StackVec::<char, 128>::from_elem('d', 2);
-    /// assert_eq!(vec.as_slice(), ['d', 'd']);
+    /// assert_eq!(&vec[..], ['d', 'd']);
     /// ```
+    #[inline]
     pub fn from_elem(elem: T, n: usize) -> Self {
         let mut vec = StackVec::<T, N>::new();
         vec.push_elem(elem, n);
@@ -484,8 +723,9 @@ impl<T: Clone, const N: usize> StackVec<T, N> {
     /// let mut  vec = StackVec::<char, 10>::new();
     /// vec.push('a');
     /// vec.push_elem('d', 2);
-    /// assert_eq!(vec.as_slice(), ['a', 'd', 'd']);
+    /// assert_eq!(&vec[..], ['a', 'd', 'd']);
     /// ```
+    #[inline]
     pub fn push_elem(&mut self, elem: T, n: usize) {
         assert!(self.remaining_capacity() >= n);
         unsafe {
@@ -513,8 +753,9 @@ impl<T: Clone, const N: usize> StackVec<T, N> {
     /// use stack_buf::{StackVec, stack_vec};
     /// let mut vec = stack_vec![10#1];
     /// vec.extend_from_slice(&[2, 3, 4]);
-    /// assert_eq!(vec.as_slice(), [1, 2, 3, 4]);
+    /// assert_eq!(&vec[..], [1, 2, 3, 4]);
     /// ```
+    #[inline]
     pub fn extend_from_slice(&mut self, other: &[T]) {
         assert!(self.remaining_capacity() >= other.len());
         unsafe {
@@ -526,6 +767,44 @@ impl<T: Clone, const N: usize> StackVec<T, N> {
             }
         }
     }
+
+    /// Resizes the `Vec` in-place so that `len` is equal to `new_len`.
+    ///
+    /// If `new_len` is greater than `len`, the `Vec` is extended by the
+    /// difference, with each additional slot filled with `value`.
+    /// If `new_len` is less than `len`, the `Vec` is simply truncated.
+    ///
+    /// This method requires `T` to implement [`Clone`],
+    /// in order to be able to clone the passed value.
+    ///
+    /// # Panics
+    ///
+    /// This function will be panic if `new_len > self.capacity()`.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use stack_buf::{StackVec, stack_vec};
+    ///
+    /// let mut vec = stack_vec![5#"hello"];
+    /// vec.resize(3, "world");
+    /// assert_eq!(&vec[..], ["hello", "world", "world"]);
+    ///
+    /// let mut vec = stack_vec![1, 2, 3, 4];
+    /// vec.resize(2, 0);
+    /// assert_eq!(&vec[..], [1, 2]);
+    /// ```
+    #[inline]
+    pub fn resize(&mut self, new_len: usize, value: T) {
+        assert!(new_len <= self.capacity());
+        let len = self.len();
+
+        if new_len > len {
+            self.push_elem(value, new_len - len);
+        } else {
+            self.truncate(new_len);
+        }
+    }
 }
 
 impl<T: Copy, const N: usize> StackVec<T, N> {
@@ -552,8 +831,9 @@ impl<T: Copy, const N: usize> StackVec<T, N> {
     /// dst.copy_from_slice(&src[2..]);
     ///
     /// assert_eq!(src, [1, 2, 3, 4]);
-    /// assert_eq!(dst.as_slice(), [3, 4]);
+    /// assert_eq!(&dst[..], [3, 4]);
     /// ```
+    #[inline]
     pub fn copy_from_slice(&mut self, src: &[T]) {
         assert!(self.remaining_capacity() >= src.len());
 
@@ -676,6 +956,16 @@ where
     }
 }
 
+impl<T, const N: usize> PartialEq<[T]> for StackVec<T, N>
+where
+    T: PartialEq,
+{
+    #[inline]
+    fn eq(&self, other: &[T]) -> bool {
+        **self == *other
+    }
+}
+
 impl<T, const N: usize> Eq for StackVec<T, N> where T: Eq {}
 
 impl<T: PartialOrd, const N: usize> PartialOrd for StackVec<T, N> {
@@ -776,6 +1066,24 @@ impl_range_index!(
     RangeToInclusive<usize>,
 );
 
+impl<T, const N: usize> Extend<T> for StackVec<T, N> {
+    #[inline]
+    fn extend<I: IntoIterator<Item = T>>(&mut self, iter: I) {
+        for elem in iter.into_iter() {
+            self.push(elem);
+        }
+    }
+}
+
+impl<T, const N: usize> FromIterator<T> for StackVec<T, N> {
+    #[inline]
+    fn from_iter<I: IntoIterator<Item = T>>(iter: I) -> Self {
+        let mut vec = StackVec::new();
+        vec.extend(iter);
+        vec
+    }
+}
+
 /// Iterate the `StackVec` with references to each element.
 ///
 /// ```
@@ -912,6 +1220,8 @@ impl<T: fmt::Debug, const N: usize> fmt::Debug for IntoIter<T, N> {
 
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
+use std::iter::FromIterator;
+use std::ptr::NonNull;
 
 #[cfg(feature = "serde")]
 impl<T: Serialize, const N: usize> Serialize for StackVec<T, N> {
